//	NPBPluginGUI.cpp - Editor dialog for plugins which supply their
//					   own gui.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#include "NPBPluginGUI.h"
#include "AEffEditor.hpp"

EditorComponent::EditorComponent(AEffect *plugin, NPBPluginGUI *parent, Colour col):
//Component(T("Editor"))
DialogWindow(T("Editor"), col)
{
	int width, height;

	Plugin = plugin;
	Parent = parent;

	addToDesktop(0);

	setVisible(true);
	setTopLeftPosition(250, 250);

	//get plugin's GUI's size
	//ERect *temprect = new ERect;
	ERect *temprect = 0;
	Plugin->dispatcher(Plugin, effEditGetRect, 0, 0, &temprect, 0.0f);
	width = temprect->right-temprect->left;
	height = temprect->bottom-temprect->top;
	setSize(width, height);
	setTopLeftPosition((Parent->getX()+6), (Parent->getY()+58));

	//attach plugin's GUI to this one
	NativeDesktopWindow* const w = getNativeWindow();
	void *temp = w->getNativeHandle();
	bool test;
	test = Plugin->dispatcher(Plugin, effEditOpen, 0, 0, temp, 0.0f)!=0;
	if(!test)
	{
		AlertWindow error(T("Plugin Error"), T("Could not open the plugin's editor."), AlertWindow::WarningIcon);
		error.addButton(T("OK"), 0);
		error.runModalLoop();
	}
	Plugin->dispatcher(Plugin, effEditIdle, 0, 0, NULL, 0.0f);

	//some plugins don't seem to fill their rect until open() is called
	if((width == 0)||(height == 0))
	{
		Plugin->dispatcher(Plugin, effEditGetRect, 0, 0, &temprect, 0.0f);
		width = temprect->right-temprect->left;
		height = temprect->bottom-temprect->top;
		setSize(width, height);
		setTopLeftPosition((Parent->getX()+6), (Parent->getY()+58));

		setSize(width, height);
	}
	Parent->setSize((width+9), (height+60));

	startTimer(10);
}

EditorComponent::~EditorComponent()
{
	if(isTimerRunning())
		stopTimer();

	Plugin->dispatcher(Plugin, effEditClose, 0, 0, NULL, 0.0f);
}

void EditorComponent::timerCallback()
{
	if(Plugin)
		Plugin->dispatcher(Plugin, effEditIdle, 0, 0, NULL, 0.0f);
}

//-------------------------------------------------------------------------
NPBPluginGUI::NPBPluginGUI(AEffect *plugin, NPBPlugin *parent, Colour col):
DialogWindow(T("NPBPluginGUI"), col)
{
	char tempstr[256];
	sprintf(tempstr, "");
	ActualEditor = 0;

	Parent = parent;

	plugin->dispatcher(plugin, effGetEffectName, 0, 0, tempstr, 0.0f);
	setName(tempstr);

	addToDesktop(windowHasDropShadow);

	setVisible(true);
	setTopLeftPosition(250, 250);

	NPBPluginGUIContent *cont = new NPBPluginGUIContent(plugin, parent, this);
	setContentComponent(cont);

	ActualEditor = new EditorComponent(plugin, this, col);
}

NPBPluginGUI::~NPBPluginGUI()
{
	delete ActualEditor;
	Parent->EditorOpen = false;
	Parent->setGUI(0);
}

void NPBPluginGUI::setMix(float val)
{
	((NPBPluginGUIContent *)getContentComponent())->setMix(val);
}

void NPBPluginGUI::setSize(const int newWidth, const int newHeight)
{
	int real_width, real_height;

	real_width = newWidth + 4; //?
	//newHeight is the height of the content component
	real_height = newHeight + 4; //?

	Component::setSize(real_width, real_height);
}

void NPBPluginGUI::resized()
{
	if(ActualEditor)
	{
		ActualEditor->toFront(false);
		setSize((getWidth()-9), (getHeight()-60));
	}

	DialogWindow::resized();
}

//-------------------------------------------------------------------------
NPBPluginGUIContent::NPBPluginGUIContent(AEffect *plugin, NPBPlugin *parent, NPBPluginGUI *win):Component()
{
	int i;
	int tempint;
	char tempstr[256];

	Plugin = plugin;
	Parent = parent;
	Win = win;

	addAndMakeVisible(Programs = new ComboBox(T("Programs")));
	Programs->addActionListener(this);
	Programs->setBounds(108, 0, 103, 20);
	if(Plugin->dispatcher(Plugin, effGetProgramNameIndexed, 0, -1, tempstr, 0.0f))
	{
		//i.e. if we can do it the easy way
		for(i=0;i<Plugin->numPrograms;i++)
		{
			Plugin->dispatcher(Plugin, effGetProgramNameIndexed, i, -1, tempstr, 0.0f);
			Programs->addItem(tempstr, i);
			sprintf(tempstr, "");
		}
		Programs->setSelectedId(Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f), false);
	}
	else
	{
		tempint = Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f);
		//the hard way - have to set each program to find it's name
		for(i=0;i<Plugin->numPrograms;i++)
		{
			Plugin->dispatcher(Plugin, effSetProgram, 0, i, tempstr, 0.0f);
			Plugin->dispatcher(Plugin, effGetProgramName, 0, 0, tempstr, 0.0f);
			Programs->addItem(tempstr, i);
			sprintf(tempstr, "");
		}
		Plugin->dispatcher(Plugin, effSetProgram, 0, tempint, tempstr, 0.0f);
		Programs->setSelectedId(Plugin->dispatcher(Plugin, effGetProgram, 0, 0, tempstr, 0.0f), false);
	}

	addAndMakeVisible(MixLabel = new Label(T("MixLabel"), T("Mix :")));
	MixLabel->setBounds(216, 0, 37, 20);

	addAndMakeVisible(Mix = new Slider(T("Mix")));
	Mix->addChangeListener(this);
	//Mix->setTextBoxProportionalSize(0.0f);
	Mix->setTextBoxStyle(Slider::NoTextBox, true, 0, 0);
	Mix->setRange(0.0, 1.0);
	Mix->setBounds(258, 0, 130, 20);
	Mix->setValue((float)Parent->getMix());
}

NPBPluginGUIContent::~NPBPluginGUIContent()
{
}

void NPBPluginGUIContent::actionListenerCallback(const String& m)
{
	if(m == Programs->getName())
	{
		Win->updateEditor();
		Plugin->dispatcher(Plugin, effSetProgram, 0, Programs->getSelectedId(), NULL, 0.0f);
	}
}

void NPBPluginGUIContent::changeListenerCallback(void *objectThatHasChanged)
{
	if(objectThatHasChanged == Mix)
		Parent->setMix((float)Mix->getValue());
}

void NPBPluginGUIContent::resized()
{
	Programs->setBoundsRelative(0.10f, 0.0f, 0.35f, 0.1f);
	Programs->setSize(Programs->getWidth(), 20);
	MixLabel->setBoundsRelative(0.46f, 0.0f, 0.15f, 0.1f);
	MixLabel->setSize(MixLabel->getWidth(), 20);
	Mix->setBoundsRelative(0.61f, 0.0f, 0.35f, 0.1f);
	Mix->setSize(Mix->getWidth(), 20);
}