///	OptionsButton.cpp - The program's File menu.
///	--------------------------------------------------------------------------
///	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
///	Copyright (C) 2005  Niall Moody
///	
///	This program is free software; you can redistribute it and/or modify
///	it under the terms of the GNU General Public License as published by
///	the Free Software Foundation; either version 2 of the License, or
///	(at your option) any later version.
///	
///	This program is distributed in the hope that it will be useful,
///	but WITHOUT ANY WARRANTY; without even the implied warranty of
///	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///	GNU General Public License for more details.
///	
///	You should have received a copy of the GNU General Public License
///	along with this program; if not, write to the Free Software
///	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
///	
///	--------------------------------------------------------------------------
///	
///	You can e-mail me at niallmoody@yahoo.co.uk if you want.
///	
///	--------------------------------------------------------------------------

#include "../NPBWindow.h"
#include "OptionsButton.h"
#include "../ComponentFactory.h"
#include "../NiallsTB.h"

//--static member variables must be declared--
String OptionsButtonComp::text = T("Start Audio");
int OptionsButtonComp::red = 0;
int OptionsButtonComp::green = 0;
int OptionsButtonComp::blue = 0;

///---------------------------------------------------------------------------
OptionsButtonComp::OptionsButtonComp(ComponentFactory *factory, Component **windowComponent):
FactoryObject(factory, windowComponent)
{
	//--Set default values to allow for the user missing out on an attribute.--
	x = 110;
	y = -31;
	xRel = -1;
	yRel = -1;
	width = 100;
	height = 25;
	widthRel = -1;
	heightRel = -1;

	//Register this object with the factory.
	factory->registerObject(this, String("optionsButton"));

	//Register all it's attributes with the factory.
	factory->registerStringAttribute(this, String("text"), &OptionsButtonComp::setText);
	factory->registerIntAttribute(this, String("colour_red"), &OptionsButtonComp::setColourRed);
	factory->registerIntAttribute(this, String("colour_green"), &OptionsButtonComp::setColourGreen);
	factory->registerIntAttribute(this, String("colour_blue"), &OptionsButtonComp::setColourBlue);
}

///---------------------------------------------------------------------------
void OptionsButtonComp::create()
{
	NPBWindow *window;
	NiallsTB *tempComp;
	Colour tempcol(red, green, blue);

	window = static_cast<NPBWindow *>(factory->getWindow());

	window->addAndMakeVisible(tempComp = new NiallsTB(T("Options"),
														T("Audio options etc.")));
	tempComp->setBackgroundColours(tempcol, Colours::white);
	tempComp->setButtonText(text);
	tempComp->setName(T("Options"));
	tempComp->addActionListener(window);

	*windowComponent = tempComp;
}

///---------------------------------------------------------------------------
void OptionsButtonComp::setText(String val)
{
	text = val;
}

///---------------------------------------------------------------------------
void OptionsButtonComp::setColourRed(int val)
{
	red = val;
}

///---------------------------------------------------------------------------
void OptionsButtonComp::setColourGreen(int val)
{
	green = val;
}

///---------------------------------------------------------------------------
void OptionsButtonComp::setColourBlue(int val)
{
	blue = val;
}
